
#ifndef DUNE_REDUCEDROD_MICROSTRUCTUREDRODGRID_HH
#define DUNE_REDUCEDROD_MICROSTRUCTUREDRODGRID_HH 

#include <dune/grid/yaspgrid.hh>
#include <dune/grid/onedgrid.hh>
#include <dune/functions/functionspacebases/powerbasis.hh>
#include <dune/functions/functionspacebases/lagrangebasis.hh>

using namespace Dune;
using namespace Functions::BasisBuilder;


class MicrostructuredRodGrid {

public:

  	//static const int orderFE_Ce = 1; 
	  static const int dim = 3;
	  static const int nCompo = 3;

  	using CellGridType = YaspGrid< dim, EquidistantOffsetCoordinates< double, dim>>;
  	using Rod1DGridType = OneDGrid;
  	using Rod3DGridType = CellGridType;

    using IsometricRodBasisType = Functions::LagrangeBasis<Rod1DGridType::LeafGridView, 1>;

protected:

	  const double width_;
  	const double len_;
  	const double eps_;
  	const double h_;       //thickness h könnte auch in plotmethoden vorkommen und Querschnitt bleibt skaliert

  	const std::array<int,dim> nE_Ce_;
  	const FieldVector<double,dim> bboxL_Ce_;
  	const FieldVector<double,dim> bboxR_Ce_;

  	const int nE_R1D_;

  	const std::array<int,dim> nE_R3D_;
  	const FieldVector<double,dim> bboxL_R3D_;
  	const FieldVector<double,dim> bboxR_R3D_;

  	const CellGridType grid_Ce_;
  	const Rod1DGridType grid_R1D_;
  	const Rod3DGridType grid_R3D_;

    const IsometricRodBasisType basis_IsoRod_;



public: 

	// Constructor
	MicrostructuredRodGrid(	const double width, 
						  	const double len, 
						  	const double eps, 
						  	const double h,
						  	const std::array<int,dim> nE_Ce):  
        width_(width), 
        len_(len), 
        eps_(eps), 
        h_(h), 
        
        nE_Ce_(nE_Ce),
        bboxL_Ce_({0.0, -width/2.0, -width/2.0}),
        bboxR_Ce_({1.0, width/2.0, width/2.0}),

        nE_R1D_(int( (len *nE_Ce[0])/ eps) ),

        
        //nE_R3D_({nE_R1D_, int(double(nE_Ce[1])/h), int(double(nE_Ce[2])/h)}),
        nE_R3D_({nE_R1D_, nE_Ce[1], nE_Ce[2]}),
        bboxL_R3D_({0, h*bboxL_Ce_[1], h*bboxL_Ce_[2] }),
        bboxR_R3D_({len, h*bboxR_Ce_[1], h*bboxR_Ce_[2] }),

        grid_Ce_(bboxL_Ce_, bboxR_Ce_, nE_Ce),
        grid_R1D_(nE_R1D_, 0, len),
        grid_R3D_(bboxL_R3D_, bboxR_R3D_,  nE_R3D_),

        basis_IsoRod_(grid_R1D_.leafGridView())

    {}


    // Getter
    double getWidth() {return width_;}
    double getLen() {return len_;}
    double getEps() {return eps_;}
    double getH() {return h_;}

    std::array<int,dim> getNECell() {return nE_Ce_;}
    FieldVector<double,dim> getBboxLCell() {return bboxL_Ce_;}
    FieldVector<double,dim> getBboxRCell() {return bboxR_Ce_;}

    const int getNERod1D(){return nE_R1D_;}

    std::array<int,dim> getNERod3D() {return nE_R3D_;}
    FieldVector<double,dim> getBboxLRod3D() {return bboxL_R3D_;}
    FieldVector<double,dim> getBboxRRod3D() {return bboxR_R3D_;}

    const CellGridType & getCellGrid() {return grid_Ce_;}
    const Rod1DGridType & getRod1DGrid() {return grid_R1D_;}
    const Rod3DGridType & getRod3DGrid() {return grid_R3D_;}

    const IsometricRodBasisType* getIsometricRodBasis() {return &basis_IsoRod_;}

    //return gridViews

    //Rod1DGridType copyRod1DGrid() {Rod1DGridType grid = grid_R1D_; return grid;}

}; // end class

#endif